## Universidad de Valparaiso
## 12 Sept 2023
## MFA-NIPALS data(wine)

library(ade4)
library(FactoMineR)
library(missMDA)

data("wine")

new.wine <- wine[,c(-1,-2)]
dim(new.wine)

MFAclassic <- MFA(new.wine, group=c(5,3,10,9,2), type=c(rep("s",5)),
              name.group=c("olf","vis","olfag","gust","ens"))


FactoMineR::plot.MFA(MFAclassic,choix = "ind",graph.type = "classic")

FactoMineR::plot.MFA(MFAclassic,choix = "var",graph.type = "classic")

MFAclassic$eig

## missing data.

set.seed(10)

## fmd(Xo,a) ## Xo data with NA, a: percentage of NA

fmd <- function(Xo,a){
  X. <- as.matrix(Xo)
  n <- nrow(X.); p <- ncol(X.); N <- n*p
  m <- sample(N, round(a*N,0)) ; d <- length(m)
  for(j in 1:d){
    X.[m[j]] <- NA
  }
  return(X.)
}

wine.na07 <- fmd(new.wine,0.07) ## 7% of NA


### MFA-NIPALS

## MFA.nipals(data,group,it)


MFA.nipals <- function(data, group, it, ncp) {
  ## group: is a vector with the number variables per group.
  ## it: is the number iterations in NIPALS.
  ## ncp: number of axes to keep.
  
  nipalsg1 <- ade4::nipals(data[,1:group[1]], niter=it, nf=ncp)
  nipalsg2 <- ade4::nipals(data[,(group[1]+1):sum(group[1:2])], niter=it, nf=ncp)
  nipalsg3 <- ade4::nipals(data[,(sum(group[1:2])+1):sum(group[1:3])], niter=it, nf=ncp)
  nipalsg4 <- ade4::nipals(data[,(sum(group[1:3])+1):sum(group[1:4])], niter=it, nf=ncp)
  nipalsg5 <- ade4::nipals(data[,(sum(group[1:4])+1):sum(group)], niter=it, nf=ncp)
  
  dataZ.na <- cbind(data[,1:group[1]]/nipalsg1$eig[1],
                    data[,(group[1]+1):sum(group[1:2])]/nipalsg2$eig[1],
                    data[,(sum(group[1:2])+1):sum(1:3)]/nipalsg3$eig[1],
                    data[,(sum(group[1:3])+1):sum(1:4)]/nipalsg4$eig[1],
                    data[,(sum(group[1:4])+1):sum(group)]/nipalsg5$eig[1])
  
  nipals.global.na <- ade4::nipals(dataZ.na,niter=it, nf=ncp)
  
  ## individual factor map
  plot(nipals.global.na$li[,1],nipals.global.na$li[,2],
       xlab="dim1",ylab="dim2",pch=19,
       main="Individual factor map with MFA-NIPALS")
  text(nipals.global.na$li[,1]+0.001,nipals.global.na$li[,2]+0.2,
       row.names(data), cex=0.7)
  abline(h=0)
  abline(v=0)
  
  ## corcircle
  
  #### circulo
  radio <- 1
  t <- seq(0, 2*pi, length.out = 100)
  
  x <- cos(t)*radio
  y <- sin(t)*radio
  
  ### corcircle
  
  plot(nipals.global.na$co[,1],nipals.global.na$co[,2],
       xlab="dim1",ylab="dim2",pch=19,
       main="Correlation circle map with MFA-NIPALS",
       ylim=c(-1.2,1.2),xlim=c(-1.1,1.1))
  text(nipals.global.na$co[,1],nipals.global.na$co[,2]+0.001,
       colnames(data),
       col=c(rep(2,group[1]),rep(3,group[2]), rep(4,group[3]), 
             rep(5,group[4]), rep(6,group[5])), cex=0.55)
  lines(x, y, lwd = 1)
  abline(h=0)
  abline(v=0)
  
  results = list(T = nipals.global.na$li, F= nipals.global.na$co,
                 lambda= nipals.global.na$eig)
  
}



MFAnipals.wine <- MFA.nipals(wine.na07, group=c(5,3,10,9,2), it=10000000,
                             ncp=29)

MFAnipals.wine$T ## coordenadas individuos
MFAnipals.wine$F ## coordenadas variables
MFAnipals.wine$lambda ## valores propios



cor(MFAclassic$ind$coord[,1], MFAnipals.wine$T[,1])
cor(MFAclassic$ind$coord[,2], MFAnipals.wine$T[,2])
cor(MFAclassic$ind$coord[,3], MFAnipals.wine$T[,3])
cor(MFAclassic$ind$coord[,4], MFAnipals.wine$T[,4])

## % varianza explicado
sum(MFAnipals.wine$lambda[1:2])/sum(MFAnipals.wine$lambda)


## Imputacion de datos con RIMFA

resImpute <- imputeMFA(wine.na07, group=c(5,3,10,9,2),
                       type=c(rep("s",5)), ncp=2, method = c("Regularized"))
                       
WineImpute <- resImpute$completeObs

MFA.WineImpute <- MFA(WineImpute, group=c(5,3,10,9,2), type=c(rep("s",5)),
                  name.group=c("olf","vis","olfag","gust","ens"))



cor(MFAclassic$ind$coord[,1], MFA.WineImpute$ind$coord[,1])
cor(MFAclassic$ind$coord[,2], MFA.WineImpute$ind$coord[,2])
cor(MFAclassic$ind$coord[,3], MFA.WineImpute$ind$coord[,3])
cor(MFAclassic$ind$coord[,4], MFA.WineImpute$ind$coord[,4])


